/*---------------------------------------------------------------
/* Split Votes  (c)1993 Ron Dippold All Rights Reserved
/* UVSplit Version 1.2
/*
/* This software separates the mailed in votes and bounces from a
/* single file for votes and a single file for bounces into several files.
/*
/*-------------------------------------------------------------*/
#define VERSION "UVSplit 1.2 Vote Processing - (c) 1993 Ron Dippold"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "uvsplit.h"

FILE *fin=NULL, *fcmd=NULL, *fout=NULL;
char votesfile[200], errorfile[200], closedfile[200], discardfile[200],
     bouncesfile[200];

char line[400];                   /* input line */
char name[400], addr[400], subj[400];/* name, address, subject */
char name2[400], addr2[400];        /* for Reply-To: */
char tag[200];                    /* tag from line */
char uin[81];                     /* user input */
int count, mail;                  /* count messages and mail */
int startisend;                   /* if start is same as end for mail */
int lmailstart, lmailend;

int dotest;                       /* test config file */
int dointer;                      /* interactive */
int forceclosed;                  /* force vote closed status */

postype start, pos, tpos;         /* start of message, current pos */
int mlines;                       /* number of lines in message */

int nvotes;                       /* number of votes running */
struct {
  char name[80];                  /* name of vote */
  int  active;                    /* active or closed */
  char file[200];                 /* file name */
  int keys;                       /* number of keywords */
  char key[KEYS][40];             /* keywords */
  int keylen[KEYS];               /* length of key */
} vote[VOTESMAX];
char fname[200];                  /* votes/bounces */

int matched[VOTESMAX];            /* matched the vote? */
int totmatched[VOTESMAX];         /* total votes matched */
int nomatched;                    /* non matching votes */

char acktext[3][20][80];          /* ack texts */
int acklen[3];                    /* length of ack */
#define SIGTEXT 0
#define MATCHTEXT 1
#define CLOSEDTEXT 2

/* define state machine states */
#define NOTHING_S    0     /* no state */
#define INVOTE_S     1     /* in vote */
int state= NOTHING_S;

#define PAGELINES ( PAGELEN-3 )

void do_exit();
int space();
int isletter();
int strlook();
void decode_nameaddr();
void replyto();


/* ------------------------------------------------------------------
/* exit cleanly
/* ------------------------------------------------------------------
*/
void do_exit( code )
int code;                         /* code to exit with */
{
  if( fin != NULL ) fclose( fin );
  if( fcmd != NULL ) fclose( fcmd );
  if( fout != NULL ) fclose( fout );
  exit( code );
}

/* ------------------------------------------------------------------
/* Check if a character is white space
/* ------------------------------------------------------------------
*/
int space( c )
char c;                           /* character to check */
{
  if( c == ' ' || c == '\t' || c == '\n' ) {
    return( 1 );
  } else {
    return( 0 );
  }
}

/* ----------------------------------------------------------------------
/*  Strip all whitespace from ends of string
/* ----------------------------------------------------------------------
*/
void stripstr( str )
char *str;                        /* string to process */
{
  int i, j;

  for( i=0; space( str[i] ); i++ );   /* skip spaces */
  for( j=0; str[i]; str[j++]=str[i++] ) {
    if( str[i] == '\n' ) break;   /* cr ends */
  }
  do {
    str[j--] = 0;                 /* zero terminate */
  } while( j>=0 && space( str[j] ) );
}


/* ------------------------------------------------------------------
/* Convert lower case to upper case
/* ------------------------------------------------------------------
*/
char upcase( c )
char c;                           /* character to convert */
{
  if( c>='a' && c<='z' ) return( c-('a'-'A'));
  return( c );
}

/* ------------------------------------------------------------------
/* is letter - check for a-z,  A-Z
/* ------------------------------------------------------------------
*/
int isletter( letter )
char letter;                      /* letter to check */
{
  if( ( letter >='a' && letter<='z' ) ||
      ( letter >='A' && letter<='Z' ) ) {
    return( 1 );
  } else {
    return( 0 );
  }
}


/* ----------------------------------------------------------------------
/*  Compare strings, ignoring case, for a specific length.  Apparently
/*  strnicmp() isn't standard.  # is treated as a wildcharacter in the
/*  first string.
/* ----------------------------------------------------------------------
*/
int strlook( str1, str2, len )
char *str1;                       /* first string */
char *str2;                       /* second string */
int len;                          /* length to compare for */
{
  while ( len-- )
    if( *str1 != '#' )
      if( ( *str1++ | 0x20 ) != ( *str2++ | 0x20 ) )
        return( 1 );
  return(0);
}


/* ----------------------------------------------------------------------
/*  This does the tricky work of decoding which is the name and which is
/*  the address.  Used for both From: and Reply-To:
/* ----------------------------------------------------------------------
*/
void decode_nameaddr( raw, name, addr )
char *raw;                        /* the raw name+address */
char *name;                       /* resulting name */
char *addr;                       /* resulting address */
{

  int i, j, baddr, deep;
  char temp[80], c;
  int state;

#define FIND_NEXT 0
#define IN_ADDR 1
#define IN_NAME 2

  addr[0]=0; name[0]=0;           /* nothing in these yet */
  baddr = FALSE;                  /* check for bracketed address */
  for( i=0; raw[i]; i++ ) {
    if( raw[i] == '<' ) {
      baddr = TRUE;
    }
  }

  i = 0;                          /* start at beginning */
  deep=0;                         /* only one level deep */
  state = FIND_NEXT;
  while( raw[i] ) {
    c = raw[i++];
    switch( state ) {
      case FIND_NEXT:             /* looking for anything */
        j = 0;
        switch( c ) {
          case '\n':              /* end of line */
          case ' ':               /* space */
          case '\t':              /* tab */
            break;
          case '(':              /* start of name */
            state = IN_NAME;
            deep = 1;
            break;
          case '<':               /* start of address */
            state = IN_ADDR;
            break;
          default:                /* other */
            if( baddr ) {         /* bracketed address exists */
              state = IN_NAME;    /* then this must be the name */
              deep = 1;
            } else {
              state = IN_ADDR;
            }
            if( c != '"' ) temp[j++] = c; /* save character */
        }
        break;

        case IN_ADDR:
          if( c == '>' || c == '(' || c=='\n' ) { /* end of address */
            while( j>0 && temp[j-1] == ' ' ) j--; /* erase any end spaces */
            temp[j] = '\0';       /* terminate string */
            temp[70] = '\0';      /* just in case... */
            strcpy( addr, temp ); /* and copy */
            state = FIND_NEXT;
            if( c=='(' ) i--;
          } else {
            temp[j++]=c;
          }
          break;

        case IN_NAME:
         if( c==')') --deep;
         if( deep == 0 || c == '<' || c=='\n' ) {  /* end of name */
           deep = 0;
           while( j>0 && temp[j-1] == ' ' ) j--; /* erase any end spaces */
            if( temp[j-1] == '"' ) j--; /* strip last quote */
            temp[j] = '\0';       /* terminate string */
            temp[70] = '\0';      /* just in case... */
            strcpy( name, temp ); /* and copy */
            state = FIND_NEXT;
            if( c=='<' ) i--;
          } else {
            if(c=='(') deep++;
            temp[j++]=c;
          }
          break;

    }                             /* switch */
  }                               /* while raw[i] */
}


/* ---------------------------------------------------------------------
/* Read the configuration file - a big file!
/* ---------------------------------------------------------------------
*/
void read_config()
{
  int state, cnt, i, j;

#define INNONE 0
#define INFIX 1
#define INVOTES 2
#define INVOTE 3
#define INSIG 4
#define INMATCH 5
#define INCLOSED 6

  state = INNONE;                 /* not in anything yet */

  nvotes = 0;                     /* no votes yet */
  cnt = 0;                        /* no lines */
  votesfile[0] = errorfile[0] = closedfile[0] =
    bouncesfile[0] = discardfile[0] = 0; /* no file names */
  for( i=0; i<3; acktext[i++][0][0] = 0 ); /* no text */
  for( i=0; i<3; acklen[i++] = 0 ); /* no text */

  if ( !( fin = fopen( CONFIGFILE, UV_OPEN_READ ))) {
    printf( "Can't open config file %s for input\n", CONFIGFILE );
    do_exit( 50 );
  }
  for( ;; ) {
    if( !fgets( line, 200, fin ) ) {
      break;
    }
    cnt++;
    if( line[0] == '#' ) continue;
    for( i=0; line[i]; i++ ) {
      if( line[i] == '\n' ) line[i]=0;
    }
    switch ( state ) {

      case INNONE:
        if( line[0] == 0 ) break; /* empty line */
        if( strlook( "[fixed files]", line, 13 ) == 0 ) {
          state = INFIX;
        } else if( strlook( "[vote info]", line, 11 ) == 0 ) {
          state = INVOTES;
        } else if( strlook( "[signature]", line, 11 ) == 0 ) {
          state = INSIG;
          j = 0;
        } else if( strlook( "[no match]", line, 10 ) == 0 ) {
          state = INMATCH;
          j = 0;
        } else if( strlook( "[closed vote]", line, 13 ) == 0 ) {
          state = INCLOSED;
          j = 0;
        } else {
          printf( "None Error line %d:\n%s", cnt, line );
          do_exit( 51 );
        }
        break;
      case INFIX:
        if( line[0] == 0 ) break; /* empty line */
        if( strlook( "[end]", line, 5 ) == 0 ) {
          state = INNONE;
        } else if( strlook( "VOTESFILE=", line, 10 ) == 0 ) {
          for( i=10; line[i]==' '; i++ ); /* skip spaces */
          strcpy( votesfile, &line[i] );
        } else if( strlook( "BOUNCESFILE=", line, 12 ) == 0 ) {
          for( i=12; line[i]==' '; i++ ); /* skip spaces */
          strcpy( bouncesfile, &line[i] );
        } else if( strlook( "ERRORFILE=", line, 10 ) == 0 ) {
          for( i=10; line[i]==' '; i++ ); /* skip spaces */
          strcpy( errorfile, &line[i] );
        } else if( strlook( "CLOSEDFILE=", line, 11 ) == 0 ) {
          for( i=11; line[i]==' '; i++ ); /* skip spaces */
          strcpy( closedfile, &line[i] );
        } else if( strlook( "DISCARDFILE=", line, 12 ) == 0 ) {
          for( i=12; line[i]==' '; i++ ); /* skip spaces */
          strcpy( discardfile, &line[i] );
        } else {
          printf( "Fixed Error line %d:\n%s", cnt, line );
          do_exit( 52 );
        }
        break;
      case INVOTES:
        if( line[0] == 0 ) break; /* empty line */
        if( strlook( "[end]", line, 5 ) == 0 ) {
          state = INNONE;
        } else if( strlook( "VOTE=", line, 5 ) == 0 ) {
          for( i=5; line[i]==' '; i++ ); /* skip spaces */
          strcpy( vote[nvotes].name, &line[i] );
          state = INVOTE;
          vote[nvotes].keys=0;
          vote[nvotes].active=TRUE;
          vote[nvotes].file[0]=0;
        } else {
          printf( "Votes Error line %d:\n%s", cnt, line );
          do_exit( 53 );
        }
        break;
      case INVOTE:
        if( strlook( "STATUS=", line, 7 ) == 0 ) {
          for( i=7; line[i]==' '; i++ ); /* skip spaces */
          if( ( line[i] | 0x20 ) == 'a' ) {
            vote[nvotes].active = TRUE;
          } else if ( ( line[i] | 0x20 ) == 'c' ) {
            vote[nvotes].active = FALSE;
          } else {
            printf( "Status Error line %d:\n%s", cnt, line );
            do_exit( 54 );
          }
        } else if( strlook( "FILE=", line, 5 ) == 0 ) {
          for( i=5; line[i]==' '; i++ ); /* skip spaces */
          strcpy( vote[nvotes].file, &line[i] );
        } else if( strlook( "FILE=", line, 5 ) == 0 ) {
          for( i=5; line[i]==' '; i++ ); /* skip spaces */
          strcpy( vote[nvotes].file, &line[i] );
        } else if( strlook( "KEYWORDS=", line, 9 ) == 0 ) {
          i = 9;
          do {
            while( line[i]==' ' ) i++; /* skip spaces */
            j = 0;
            if( line[i] != 0 ) {
              while( line[i] != 0 && line[i] != ' ' ) {
                vote[nvotes].key[vote[nvotes].keys][j++] = line[i++];
              }
              vote[nvotes].key[vote[nvotes].keys][j] = 0;
              vote[nvotes].keylen[vote[nvotes].keys] = j;
              vote[nvotes].keys++;
            }
          } while( line[i] != 0 );
          nvotes++;               /* number of votes */
          state = INVOTES;
        } else {
          printf( "Vote Error line %d:\n%s", cnt, line );
          do_exit( 54 );
        }
        break;
      case INSIG:
      case INMATCH:
      case INCLOSED:
        if( line[0] == 0 ) {
          line[0] =' ';
          line[1] = 0;
        }
        if( strlen( line )>=80 ) {
          printf( "Error line %d too long:\n%s", cnt, line );
          do_exit( 55 );
        }
        if( strlook( "[end]", line, 5 ) == 0 ) {
          state = INNONE;
        } else {
          if( j == 20 ) {
            printf( "Error line %d - too many lines in text:\n%s", cnt, line );
            do_exit( 55 );
          }
          if( state == INSIG ) {
            strcpy( acktext[SIGTEXT][j++], line );
            acklen[SIGTEXT]=j;
          }
          if( state == INMATCH ) {
            strcpy( acktext[MATCHTEXT][j++], line );
            acklen[MATCHTEXT]=j;
          }
          if( state == INCLOSED ) {
            strcpy( acktext[CLOSEDTEXT][j++], line );
            acklen[CLOSEDTEXT]=j;
          }
        }
        break;
    }                             /* switch */
  }                               /* for ( ;; ) */

  if( votesfile[0] == 0 ) {
    printf( " Error - no VOTESFILE= given\n" );
    do_exit( 56 );
  }
  if( errorfile[0] == 0 ) {
    printf( " Error - no ERRORFILE= given\n" );
    do_exit( 56 );
  }
  if( closedfile[0] == 0 ) {
    printf( " Error - no CLOSEDFILE= given\n" );
    do_exit( 56 );
  }
  if( discardfile[0] == 0 ) {
    printf( " Error - no DISCARDFILE= given\n" );
    do_exit( 56 );
  }
  if( acklen[SIGTEXT] == 0 ) {
    printf( " Error - no [signature] given\n" );
    do_exit( 56 );
  }
  if( acklen[MATCHTEXT] == 0 ) {
    printf( " Error - no [no match] given\n" );
    do_exit( 56 );
  }
  if( acklen[CLOSEDTEXT] == 0 ) {
    printf( " Error - no [closed vote] given\n" );
    do_exit( 56 );
  }
  if( nvotes == 0 ) {
    printf( " Error - no [vote info] given\n" );
    do_exit( 56 );
  }
  for( i=0; i<nvotes; i++ ) {
    if( vote[i].name[0] == 0 ) {
      printf( " Error on group %d - no NAME=\n", i+1 );
      do_exit( 57 );
    }
    if( vote[i].file[0] == 0 ) {
      printf( " Error on group %d - no FILE=\n", i+1 );
      do_exit( 57 );
    }
    for( j=0; vote[i].file[j] && vote[i].file[j]!='*'; j++ );
    if( vote[i].file[j]!='*' ) {
      printf( "Error on group %d - FILE= contains no '*'\n", i+1 );
      do_exit( 57 );
    }
    if( vote[i].keys == 0 ) {
      printf( " Error on group %d - no KEYWORDS=\n", i+1 );
      do_exit( 58 );
    }
  }


  if( dotest ) {
    printf( "\nVOTE FILE: %s\n", votesfile );
    printf( "BOUNCES FILE: %s\n", bouncesfile );
    printf( "ERROR FILE: %s\n", errorfile );
    printf( "CLOSED FILE: %s\n", closedfile );
    printf( "DISCARD FILE: %s\n", discardfile );
    puts( "\n[signature]:" );
    for( i=0; i<acklen[SIGTEXT]; i++ ) puts( acktext[SIGTEXT][i] );
    puts( "\n[no match]:" );
    for( i=0; i<acklen[MATCHTEXT]; i++ ) puts( acktext[MATCHTEXT][i] );
    puts( "\n[closed vote]:" );
    for( i=0; i<acklen[CLOSEDTEXT]; i++ ) puts( acktext[CLOSEDTEXT][i] );
    for( i=0; i<nvotes; i++ ) {
      printf( "\nVote #%d:\n", i+1 );
      printf( "  Name: %s\n", vote[i].name );
      printf( "  Active: %c\n", vote[i].active ? 'Y' :'N' );
      printf( "  File: %s\n", vote[i].file );
      printf( "  Keywords: \n" );
      for( j=0; j< vote[i].keys; j++ ) {
        printf( "    %s\n", vote[i].key[j] );
      }
    }
    do_exit( 0 );
  }
}


/* ---------------------------------------------------------------------
/* Check a line for keywords
/* ---------------------------------------------------------------------
*/
checkline()
{
  int match, len, v, i, j;
  static int keymatch[KEYS];

  len = strlen( line );
  for( v=0; v<nvotes; v++ ) {     /* for each vote */
    for( j=0; j<KEYS; keymatch[j++]=0 ); /* zero out key matches */
    for( i=0; line[i]; i++ ) {    /* go over the whole line */
      for( j=0; j<vote[v].keys; j++ ) { /* check each key */
        if( len > vote[v].keylen[j]+i ) { /* don't check if too long */
          if( strlook( vote[v].key[j], &line[i], vote[v].keylen[j] ) == 0 ) {
            keymatch[j] = TRUE;
          }
        }
      }                           /* for j */
    }                             /* for i */
    for( i=TRUE, j=0; j<vote[v].keys; j++ ) {
      if( !keymatch[j] ) {
        i = FALSE;
        break;
      }
    }
    if( i ) {
      matched[v] = TRUE;
    }
  }                               /* for v */
}

/* ---------------------------------------------------------------------
/* mail an error message
/* ---------------------------------------------------------------------
*/
void err_mail( errtext )
int errtext;
{
#ifdef DOVOTEACK
  int i, j, k, doquote;
  static char line2[201];         /* input line */
  static char fname[20];

  if( addr[0] == 0 ) {            /* can't do anything with no address */
    return;
  }

  /* generate mail file name */
  sprintf( fname, "acq.%c%c%c",  (mail/( 26*26 )) +'a',
      (( mail/26 ) % 26 ) +'a', ( mail % 26 ) + 'a' );
  mail++;

  if ( !( fout = fopen( fname, UV_OPEN_WRITE ))) {
    printf( "Can't open %s for output\n", fname );
    do_exit( 2 );
  }

  /* now make a command to mail this thing */
  for( i=0, j=0; MAILCMD[i]; i++ ) {
    if( MAILCMD[i] != '#' ) {
      line[j++] = MAILCMD[i];
    } else {
      doquote = 0;
      switch ( MAILCMD[++i] ) {
        case 'S':
          doquote = 1;
          line[j++] = '"';
        case 's':
          for( k=0; VOTEACKSUB[k]; k++ ) line[j++] = VOTEACKSUB[k];
          if( doquote ) line[j++] = '"';
          break;
        case 'V':
          doquote = 1;
          line[j++] = '"';
        case 'v':
          for( k=0; addr[k]; k++ ) {
            word l;
            for( l=0; l<strlen( ADDBADCHAR ); l++ ) {
              if( addr[k] == ADDBADCHAR[l] ) {
                line[j++] = '\\'; break;
              }
            }
            line[j++] = addr[k];
          }
          if( doquote ) line[j++] = '"';
          break;
        case 'f':
          for( k=0; fname[k]; k++ ) line[j++] = fname[k];
          break;
        default:
          printf( "  Error in mail text string:\n   %s\n", MAILCMD );
          do_exit( 12 );
          break;
      }  /* switch */
    }  /* # */
  }  /* for */
  line[j++]='\0';
  fprintf( fcmd, "%s\n", line );

  for( i=0; i<acklen[errtext]; i++ ) { /* print the error text */
    fprintf( fout, "%s\n", acktext[errtext][i] );
  }

  tpos = ftell( fin );        /* needed because of start/end wierdness */
  fseek( fin, start, SEEK_SET ); /* find start of message */
  for( i=0; i<mlines; i++ ) {
    if( fgets( line2, 200, fin ) ) {
      fprintf( fout, "> %s", line2 );
    }
  }
  fseek( fin, tpos, SEEK_SET );   /* find start of message */

  for( i=0; i<acklen[SIGTEXT]; i++ ) { /* print the signature text */
    fprintf( fout, "%s\n", acktext[SIGTEXT][i] );
  }
  fclose( fout );
#endif
}


/* ---------------------------------------------------------------------
/* Copy a message to an output file
/* ---------------------------------------------------------------------
*/
void msg_copy( filename )
char *filename;
{
  int i;
  static char line2[201];       /* input line */

  if ( !( fout = fopen( filename, UV_OPEN_APPEND ))) {   /* open for append, text mode */
    printf( "Can't open file %s for output\n", filename );
    do_exit( 22 );
  }

  tpos = ftell( fin );        /* needed because of start/end wierdness */
  fseek( fin, start, SEEK_SET ); /* find start of message */
  for( i=0; i<mlines; i++ ) {
    if( fgets( line2, 200, fin ) ) {
      fprintf( fout, "%s", line2 );
    }
  }
  fclose( fout );
  fseek( fin, tpos, SEEK_SET ); /* find start of message */
}

/* ---------------------------------------------------------------------
/* Show part of a message - returns the number of lines left
/* -----------------------------------------------------------------
*/
int showmsg( begin, len )
int begin;                        /* start line in message */
int len;                          /* number of lines to show */
{
  static char str[PAGEWIDE+1];
  static char mline[200];         /* get line from file */
  postype cpos;                   /* current position */
  int lines;                      /* number of lines */
  int plines;                     /* printed lines */
  int i, j;

  cpos = ftell( fin );            /* save current position */
  fseek( fin,start, SEEK_SET );   /* start of message */

  for( lines=0; lines<begin; lines++ ) {
    fgets( mline, 200, fin );      /* lines */
  }

  for( plines=0; plines<len; ) {
    if( lines >= mlines ) {      /* past end */
      strcpy( mline, "\n" );       /* blank line */
    } else {
      fgets( mline, 200, fin );    /* lines */
    }
    lines++;                      /* read another line */
    i=0;
    do {                        /* for as many lines as needed */
      for( j=0; j<PAGEWIDE && mline[i]; j++ ) {
        if( j==PAGEWIDE-2 && mline[i]!='\n' ) { /* end of screen */
          str[j++]='\\';
          str[j++]='\n';
          break;
        }
        str[j]=mline[i++];
      }
      str[j]=0;                   /* terminate */
      printf( "%s", str );
      plines++;                   /* printed another line */
    } while ( mline[i] );
  }                               /* for plines */

  fseek( fin, cpos, SEEK_SET );   /* return to last postion */
  i = mlines-lines;
  if( i<0 ) i=0;
  return( i );
}



/* ---------------------------------------------------------------------
/* interact - this is a major routine, which allows the user to view the
/* vote and make changes to it.
/* Returns TRUE if tried to make a change, FALSE if aborted
/* -----------------------------------------------------------------
*/
int interact()
{
  static char c;
  int lnum, lleft;                /* line number */
  int doneloop, i, j;

  if( !dointer ) return( FALSE ); /* not in interactive mode */

  /* ask if we want to delve into it */
  do {
    printf( "*** C)ontinue, E)xamine [E]? " );
    fgets( uin, 80, stdin );
    stripstr( uin );
    c = upcase( uin[0] );
    if( !c ) c='E';
    if( c =='C' ) {
      printf( "\n" );
      return( FALSE );            /* continue */
    }
  } while( c!='E' );

  lnum = 1;                       /* start of message */
  for( ;; ) {
#ifdef CLRSCRN
    puts( CLRSCRN );              /* clear screen */
#else
    puts( "\n\n\n" );
#endif
    lleft = showmsg( lnum, PAGELINES ); /* show message */
    for( i=0; i<PAGEWIDE-1; uin[i++]='-' ); /* separator */
    uin[i]=0;
    if( ( i=lnum+PAGELINES ) > mlines+1 ) i = mlines+1;
    sprintf( &uin[5], "[%3d%%]", ((mlines-lleft)*100)/mlines );
    for( i=0; uin[i]; i++ );
    uin[i]='-';
    printf( "%s\n", uin );
    printf( "A: %s\nN: %s\n", addr, name );
    for( doneloop=FALSE; !doneloop; ) {
      printf( "*** C)losed vote, G)roup select, L)eave as is, T)rash Msg;  D)own, U)p [D] ? " );
      fgets( uin, 80, stdin );
      stripstr( uin );
      c = upcase( uin[0] );
      if( !c ) c = 'D';      /* default */
      switch( c ) {
        case 'C':                 /* force closed */
          for( i=0; i<nvotes; i++ ) matched[i] = FALSE;
          forceclosed = TRUE;
          return( TRUE );
          break;
        case 'L':                 /* leave as is */
          printf( "\n" );
          return( FALSE );
          break;
        case 'T':                 /* trash message */
          printf( "\n" );
          for( i=0; i<nvotes; i++ ) matched[i] = 0;
          msg_copy( discardfile );
          return( TRUE );
          break;
        case 'G':
          doneloop=TRUE;
          for( ;; ) {
            printf( "\n" );
            for( i=0; i<nvotes; i++ ) {
              printf( "  %2d) %s\n", i+1, vote[i].name );
            }
            printf( "  Group? " );
            fgets( uin, 80, stdin );
            stripstr( uin );
            if( !uin[0] ) {       /* nothing entered */
              break;
            }
            sscanf( uin, "%d", &j );
            if( j>=1 && j<=nvotes ) {
              for( i=0; i<nvotes; i++ ) matched[i] = FALSE;
              matched[j-1] = TRUE;
              return( TRUE );
            }
          }
          break;
        case 'D':                 /* go down page */
        case '+':
        case '=':
          i = ( c=='D' ) ? PAGELINES : 1;
          if( i>lleft ) i=lleft;
          lnum += i;
          doneloop=TRUE;
          break;
        case 'U':                 /* go up page */
        case '-':
          lnum-= ( c=='U' ) ? PAGELINES : 1;
          if( lnum<1 ) lnum = 1;
          doneloop=TRUE;
          break;
      }                           /* switch QS; ANV; DU */
    }                             /* for !doneloop */
  }                               /* for ( ;; ) */
}

/* ---------------------------------------------------------------------
/* Make file name for bounce or votes
/* ---------------------------------------------------------------------
*/
void make_name( fname, base, isbounce )
char *fname;                      /* resulting file name */
char *base;                       /* base file name */
int isbounce;                     /* bounce or votes? */
{
  int i, j;

  for( i=j=0; base[i]; i++ ) {
    if( base[i] == '*' ) {        /* file name */
      strcpy( &fname[j], isbounce ? "bounces":"votes" );
      while( fname[++j] );
    } else {
      fname[j++] = base[i];
    }
  }
  fname[j]='\0';
}

/* ---------------------------------------------------------------------
/* Do the actual message routing - return TRUE when done, FALSE to
/* call this whole procedure again.
/* ---------------------------------------------------------------------
*/
void route( isbounce )
int isbounce;                     /* bounce file? */
{
  int i, any, closed;

  any = 0;                        /* no matches found yet */
  closed = 0;                     /* not copied to closed file yet */
  for( i=0; i<nvotes; i++ ) {
    if( matched[i] ) any++;
  }

  if( any == 0 ) {                /* no matches found */
    if( !isbounce ) {
      printf( "\n%s (%s)\n", addr, name );
    }
    printf( "   ERROR - No group matched\n" );
    forceclosed = FALSE;
    if( !interact() ) {           /* interacted, changed */
      nomatched++;
#ifdef DOVOTEACK
      if( !isbounce ) {
        err_mail( MATCHTEXT );
      }
#endif
      msg_copy( errorfile );
      return;
    } else {
      if( forceclosed ) {
#ifdef DOVOTEACK
        if( !isbounce ) {
          err_mail( CLOSEDTEXT );
        }
#endif
        msg_copy( closedfile );
        return;
      }
    }
  }

  if( any > 1 ) {                 /* more than one match */
    if( !isbounce ) {
      printf( "\n%s (%s)\n", addr, name );
    }
    printf( "  Matched multiple groups.\n" );
    for( i=0; i<nvotes; i++ ) {
      if( matched[i] ) printf( "    %s\n", vote[i].name );
    }
    interact();
  }

  for( i=0; i<nvotes; i++ ) {
    if( matched[i] ) {
      totmatched[i]++;            /* total matched */
      if( vote[i].active ) {      /* vote active */
        make_name( fname, vote[i].file, isbounce );
        msg_copy( fname );        /* copy the message */
      } else {
        if( !isbounce ) {
          printf( "\n%s (%s)\n", addr, name );
        }
        printf( "  Closed group %s.\n", vote[i].name );
#ifdef DOVOTEACK
        if( !isbounce ) {
          err_mail( CLOSEDTEXT );
        }
#endif
        if( !closed ) {
          msg_copy( closedfile );
          closed++;
        }
      }
    }
  }
  return;
}


/* ---------------------------------------------------------------------
/* Help and exit
/* ---------------------------------------------------------------------
*/
void help()
{
  printf( "\n  uvsplit (-i) <marker>            or\n" );
  printf( "  uvsplit -t \n\n" );
  printf( "    -i if used will run uvsplit in interactive mode - suspect votes\n" );
  printf( "      will be presented to you for disposition.\n\n" );
  printf( "    <marker> is a word that will be inserted in the voting\n" );
  printf( "      files to show where splits were done.  If you want to use\n" );
  printf( "      spaces in the <marker>, surround it with quotes.\n\n" );
  printf( "    -t if used will read your configuration file, then spit out what\n" );
  printf( "      it thinks it found.  No actual processing is done!\n\n" );
  do_exit( 1 );
}

/* ---------------------------------------------------------------------
/* Do the splits
/* ---------------------------------------------------------------------
*/
void dosplit( isbounce )
int isbounce;                     /* if it's a bounce split */
{
  int i, endit, isend, isstart;

  count = 0;                      /* no votes yet */
  mail = 0;                       /* no mail sent yet */
  for( i=0; i<VOTESMAX; totmatched[i++] = 0 );
  nomatched = 0;                  /* no nonmatching votes */

  if( isbounce ) {
    if( !bouncesfile[0] ) return;
    printf( "\nProcessing bounce file\n" );
    if ( !( fin = fopen( bouncesfile, UV_OPEN_READ ))) { /* open for read, text mode */
      printf( "No bounces file %s found\n", bouncesfile );
      return;
    }
  } else {
    printf( "\nProcessing vote file\n" );
    if ( !( fin = fopen( votesfile, UV_OPEN_READ ))) { /* open for read, text mode */
      printf( "No votes file %s found\n", votesfile );
      return;
    }
  }

  /* Test error file */
  if ( !( fout = fopen( errorfile, UV_OPEN_APPEND ))) {  /* open for append, text mode */
    printf( "Can't open error file %s for output\n", errorfile );
    do_exit( 2 );
  }
  fprintf( fout, "--- UVSplit: %s\n", tag );
  fclose( fout );

  /* Test closed file */
  if ( !( fout = fopen( closedfile, UV_OPEN_APPEND ))) {  /* open for append, text mode */
    printf( "Can't open closed file %s for output\n", closedfile );
    do_exit( 2 );
  }
  fprintf( fout, "--- UVSplit: %s\n", tag );
  fclose( fout );

  /* Test discard file */
  if ( !( fout = fopen( discardfile, UV_OPEN_APPEND ))) {  /* open for append, text mode */
    printf( "Can't open discard file %s for output\n", discardfile );
    do_exit( 2 );
  }
  fprintf( fout, "--- UVSplit: %s\n", tag );
  fclose( fout );

  for( i=0; i<nvotes; i++ ) {     /* try opening all results files */
    if( vote[i].active ) {
      make_name( fname, vote[i].file, isbounce );
      if ( !( fout = fopen( fname, UV_OPEN_APPEND ))) {   /* open for append, text mode */
        printf( "Can't open file %s for output\n", fname );
        do_exit( 2 );
      }
      fprintf( fout, "--- UVSplit: %s\n", tag );
      fclose( fout );
    }
  }

#ifdef DOVOTEACK
  if( !isbounce ) {
    if ( !( fcmd = fopen( CMDFILE, UV_OPEN_WRITE ))) { /* open for write, text mode */
      printf( "Can't open %s for output\n", CMDFILE );
      do_exit( 2 );
    }
#ifdef VOTEACKPREFIX
    fprintf( fcmd, "%s\n", VOTEACKPREFIX );
#endif
  }
#endif

  /*
  /*  This is the big loop  -  Once per line
  */

  do {
    pos = ftell( fin );
    if( endit = !fgets( line, 200, fin ) ) {
      isend = TRUE; isstart = FALSE;
      strcpy( line, "[END OF FILE]" );
    } else {
      mlines++;
      isend = !strncmp( line, MAILEND, lmailend ) && lmailend;
      isstart = !strncmp( line, MAILSTART, lmailstart );
    }

    if( startisend && isstart ) { /* end or start if they're the same? */
      if( state == NOTHING_S ) {
        isstart = TRUE; isend = FALSE;
      } else {
        isstart = FALSE; isend = TRUE;
      }
    }

    if( lmailend == 0 && isstart ) {
      isend = TRUE;
    }

    if( isend && count !=0 ) {    /* end of message! Do something. */
      fprintf( stderr,"." );
#if 0
      printf( "\nMessage processed - %d lines\n", mlines );
      printf( "  Addr: %s\n", addr );
      printf( "  Name: %s\n", name );
      printf( "  Subj: %s", subj );
#endif
      route( isbounce );          /* route the message till done */
    }

    if( isstart ) {
      count++;
      addr[0] = name[0] = subj[0] = 0; /* zero it out */
      mlines = 0;                 /* no lines in message yet */
      start = pos;                /* save start of message position */
      for( i=0; i<nvotes; matched[i++]=FALSE ); /* zero votes matched */
    }

    if( count != 0 ) {            /* check the line */
      checkline();
    }

    if( strncmp( line, "From:", 5 ) == 0 ) {
      decode_nameaddr( &line[6], name, addr );
    }
    if( strncmp( line, "Reply-To:", 9 ) == 0 ) {
      decode_nameaddr( &line[10], name2, addr2 );
      if( name2[0] != 0 ) {           /* if a name, copy it */
        strcpy( name, name2 );
      }
      if( addr2[0] != 0 ) {           /* if an address, copy it */
        strcpy( addr, addr2 );
      }
    }
    if( strncmp( line, "Subject:", 8 ) == 0 ) {
      strcpy( subj, &line[9] );
    }

  } while ( !endit );

#ifdef DOVOTEACK
#ifdef VOTEACKSUFFIX
  if( !isbounce ) {
    fprintf( fcmd, "%s\n", VOTEACKSUFFIX );
  }
#endif
#endif

  fclose( fin );
  if( !isbounce ) {
    printf( "\n\nAll Done... %d votes processed.\n", count );
    for( i=0; i<nvotes; i++ ) {
      printf( "  %4d matches: %s (%s)\n", totmatched[i], vote[i].name,
        vote[i].active ? "ACTIVE" : "CLOSED" );
    }
  } else {
    printf( "\n\nAll Done... %d bounces processed.\n", count );
  }
  printf( "  %4d unmatched messages.\n", nomatched );
}



/* ---------------------------------------------------------------------
/* Main procedure - drive all the other procedures
/* ---------------------------------------------------------------------
*/
int main( argc, argv )
int argc;                         /* number of arguments to uvvote */
char **argv;                      /* arguments to uvvote */
{
  int i, vidx;

  printf( "\n%s\n", VERSION );

  if( argc == 1) help();
  vidx = 1;
  dotest = FALSE;
  dointer = FALSE;
  if( argv[vidx][0] == '-' ) {    /* option! */
    if( argv[vidx][1] == 't' ) {  /* test */
      if( argc != 2 ) help();
      dotest = TRUE;
    } else if ( argv[vidx][1] == 'i' ) { /* interactive */
      dointer = TRUE;
      vidx++;
    }
  }
  if( argc-1 != vidx ) help();

  lmailstart = strlen( MAILSTART );
  if( lmailstart == 0 ) {
    fprintf( stderr, "Error: MAILSTART must be set in uvsplit.h\n" );
  }
  lmailend = strlen( MAILEND );
  startisend = FALSE;
  if( lmailstart == lmailend ) {
    if( strncmp( MAILSTART, MAILEND, lmailstart ) == 0 ) {
      startisend = TRUE;
    }
  }

  strcpy( tag, argv[vidx] );

  read_config();

  dosplit( TRUE );                /* bounce split */
  dosplit( FALSE );               /* regular split */


  do_exit(0);
}
